from __future__ import annotations

import logging
import os
from collections.abc import Sequence
from pathlib import Path
from typing import Any

from checkov.common.util.http_utils import request_wrapper

from checkov.common.bridgecrew.platform_integration import bc_integration

from checkov.common.util.tqdm_utils import ProgressBar

SLEEP_DURATION = 2
MAX_SLEEP_DURATION = 60


class Scanner:
    def __init__(self, pbar: ProgressBar | None = None, root_folder: str | Path | None = None) -> None:
        self._base_url = bc_integration.api_url
        self.bc_cli_scan_api_url = f"{self._base_url}/api/v1/vulnerabilities/cli/scan"
        if pbar:
            self.pbar = pbar
        else:
            self.pbar = ProgressBar('')
            self.pbar.turn_off_progress_bar()
        self.root_folder = root_folder

    def scan(self) -> Sequence[dict[str, Any]] | None:
        """run SCA package scan and poll scan results"""
        if not self.run_scan():
            return None
        self.poll_scan_result()
        return []

    def run_scan(self) -> bool:
        logging.info("Start to scan package files.")

        request_body = {
            "branch": "",
            "commit": "",
            "path": os.path.join(bc_integration.repo_path, '') if bc_integration.repo_path else "",
            "repoId": bc_integration.repo_id,
            "id": bc_integration.timestamp,
            "repositoryId": ""
        }

        response = request_wrapper(
            "POST", self.bc_cli_scan_api_url,
            headers=bc_integration.get_default_headers("POST"),
            json=request_body,
            should_call_raise_for_status=True
        )

        response_json = response.json()

        if not response_json["startedSuccessfully"]:
            logging.info("Failed to run package scanning.")
            return False
        return True

    def poll_scan_result(self) -> dict[str, Any]:
        return {}



from typing import Optional, Any, Dict

from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.checks_infra.solvers.attribute_solvers.base_attribute_solver import BaseAttributeSolver
from checkov.common.util.type_forcers import force_int


class NumberOfWordsEqualsAttributeSolver(BaseAttributeSolver):
    operator = Operators.NUMBER_OF_WORDS_EQUALS  # noqa: CCE003  # a static attribute

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:
        vertex_attr = vertex.get(attribute)  # type:ignore[arg-type]  # due to attribute can be None

        if not isinstance(vertex_attr, str):
            return False
        words = vertex_attr.split()
        value_numeric = force_int(self.value)

        return len(words) == value_numeric



from __future__ import annotations
from typing import Any
import jmespath

from checkov.common.images.image_referencer import Image
from checkov.common.util.consts import START_LINE, END_LINE


class CircleCIProvider:
    __slots__ = ("workflow_config", "file_path")

    def __init__(self, workflow_config: dict[str, Any], file_path: str) -> None:
        self.file_path = file_path
        self.workflow_config = workflow_config

    def extract_images_from_workflow(self) -> list[Image]:
        images: list[Image] = []

        keywords = (
            "jobs.*.docker[].{image: image, __startline__: __startline__, __endline__:__endline__}",
            "executors.*.docker[].{image: image, __startline__: __startline__, __endline__:__endline__}",
        )
        for keyword in keywords:
            results = jmespath.search(keyword, self.workflow_config)
            if not results:
                continue
            for result in results:
                image_name = result.get("image")
                if image_name:
                    images.append(
                        Image(
                            file_path=self.file_path,
                            name=image_name,
                            start_line=result[START_LINE],
                            end_line=result[END_LINE]
                        )
                    )
        return images
